using System;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;

class XmlViewForm : Form
{
    GroupBox DocumentGB;
    TextBox Source;
    Button LoadButton;
    ImageList NodeImages;
    TreeView XmlView;

    XmlViewForm ()
    {
        // Initialize the form's properties
        Text = "XML Viewer";
        ClientSize = new System.Drawing.Size (488, 422);

        // Instantiate the form's controls
        DocumentGB = new GroupBox ();
        Source = new TextBox ();
        LoadButton = new Button ();
        XmlView = new TreeView ();

        // Initialize the controls
        Source.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        Source.Location = new System.Drawing.Point (16, 24);
        Source.Size = new System.Drawing.Size (336, 24);
        Source.TabIndex = 0;
        Source.Name = "Source";

        LoadButton.Anchor = AnchorStyles.Top | AnchorStyles.Right;
        LoadButton.Location = new System.Drawing.Point (368, 24);
        LoadButton.Size = new System.Drawing.Size (72, 24);
        LoadButton.TabIndex = 1;
        LoadButton.Text = "Load";
        LoadButton.Click += new System.EventHandler (OnLoadDocument);

        DocumentGB.Anchor =
            AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
        DocumentGB.Location = new Point (16, 16);
        DocumentGB.Size = new Size (456, 64);
        DocumentGB.Text = "Document";
        DocumentGB.Controls.Add (Source);
        DocumentGB.Controls.Add (LoadButton);

        NodeImages = new ImageList ();
        NodeImages.ImageSize = new Size (12, 12);
        NodeImages.Images.AddStrip (new Bitmap (GetType (), "Buttons"));
        NodeImages.TransparentColor = Color.White;

        XmlView.Anchor = AnchorStyles.Top | AnchorStyles.Bottom |
            AnchorStyles.Left | AnchorStyles.Right;
        XmlView.Location = new System.Drawing.Point (16, 96);
        XmlView.Size = new System.Drawing.Size (456, 308);
        XmlView.ImageList = NodeImages;
        XmlView.TabIndex = 2;
        XmlView.Name = "XmlView";

        // Add the controls to the form
        Controls.Add (DocumentGB);
        Controls.Add (XmlView);
    }

    void OnLoadDocument (object sender, EventArgs e)
    {
        try {
            XmlDocument doc = new XmlDocument ();
            doc.Load (Source.Text);
            XmlView.Nodes.Clear ();
            AddNodeAndChildren (doc.DocumentElement, null);
        }
        catch (Exception ex)  {
            MessageBox.Show (ex.Message);
        }
    }

    void AddNodeAndChildren (XmlNode xnode, TreeNode tnode)
    {
        TreeNode child = AddNode (xnode, tnode);

        if (xnode.Attributes != null) {
            foreach (XmlAttribute attribute in xnode.Attributes)
                AddAttribute (attribute, child);
        }

        if (xnode.HasChildNodes) {
            foreach (XmlNode node in xnode.ChildNodes)
                AddNodeAndChildren (node, child);
        }
    }

    TreeNode AddNode (XmlNode xnode, TreeNode tnode)
    {
        string text = null;
        TreeNode child = null;

        TreeNodeCollection tnodes = (tnode == null) ?
            XmlView.Nodes : tnode.Nodes;

        switch (xnode.NodeType) {

        case XmlNodeType.Element:
        case XmlNodeType.Document:
            tnodes.Add (child = new TreeNode (xnode.Name, 0, 0));
            break;

        case XmlNodeType.Text:
            text = xnode.Value;
            if (text.Length > 128)
                text = text.Substring (0, 128) + "...";
            tnodes.Add (child = new TreeNode (text, 2, 2));
            break;

        case XmlNodeType.CDATA:
            text = xnode.Value;
            if (text.Length > 128)
                text = text.Substring (0, 128) + "...";
            text = String.Format ("<![CDATA]{0}]]>", text);
            tnodes.Add (child = new TreeNode (text, 3, 3));
            break;

        case XmlNodeType.Comment:
            text = String.Format ("<!--{0}-->", xnode.Value);
            tnodes.Add (child = new TreeNode (text, 4, 4));
            break;

        case XmlNodeType.XmlDeclaration:
        case XmlNodeType.ProcessingInstruction:
            text = String.Format ("<?{0} {1}?>", xnode.Name,
                xnode.Value);
            tnodes.Add (child = new TreeNode (text, 5, 5));
            break;

        case XmlNodeType.Entity:
            text = String.Format ("<!ENTITY {0}>", xnode.Value);
            tnodes.Add (child = new TreeNode (text, 6, 6));
            break;

        case XmlNodeType.EntityReference:
            text = String.Format ("&{0};", xnode.Value);
            tnodes.Add (child = new TreeNode (text, 7, 7));
            break;

        case XmlNodeType.DocumentType:
            text = String.Format ("<!DOCTYPE {0}>", xnode.Value);
            tnodes.Add (child = new TreeNode (text, 8, 8));
            break;

        case XmlNodeType.Notation:
            text = String.Format ("<!NOTATION {0}>", xnode.Value);
            tnodes.Add (child = new TreeNode (text, 9, 9));
            break;

        default:
            tnodes.Add (child =
                new TreeNode (xnode.NodeType.ToString (), 1, 1));
            break;
        }
        return child;
    }

    void AddAttribute (XmlAttribute attribute, TreeNode tnode)
    {
        string text = String.Format ("{0}={1}", attribute.Name,
            attribute.Value);
        tnode.Nodes.Add (new TreeNode (text, 1, 1));
    }

    static void Main () 
    {
        Application.Run (new XmlViewForm ());
    }
}